@@ -187,7 +187,7 @@ type DispatchInput struct {
 // WorkflowDispatchEvent is event on dispatching workflow manually.
 // https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#workflow_dispatch
 type WorkflowDispatchEvent struct {
-	// Inputs is map from input names to input attributes.
+	// Inputs is map from input names to input attributes. Keys are in lower case since they are case insensitive.
 	Inputs map[string]*DispatchInput
 	// Pos is a position in source.
 	Pos *Pos
@@ -230,6 +230,8 @@ const (
 // WorkflowCallEventInput is an input configuration of workflow_call event.
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callinputs
 type WorkflowCallEventInput struct {
+	// Name is a name of the input.
+	Name *String
 	// Description is a description of the input.
 	Description *String
 	// Default is a default value of the input. Nil means no default value.
@@ -251,6 +253,8 @@ func (i *WorkflowCallEventInput) IsRequired() bool {
 // WorkflowCallEventSecret is a secret configuration of workflow_call event.
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callsecrets
 type WorkflowCallEventSecret struct {
+	// Name is a name of the secret.
+	Name *String
 	// Description is a description of the secret.
 	Description *String
 	// Required represents if the secret is required or optional. When this value is nil, it means optional.
@@ -261,6 +265,8 @@ type WorkflowCallEventSecret struct {
 // WorkflowCallEventOutput is an output configuration of workflow_call event.
 // https://docs.github.com/en/actions/using-workflows/reusing-workflows#using-outputs-from-a-reusable-workflow
 type WorkflowCallEventOutput struct {
+	// Name is a name of the output.
+	Name *String
 	// Description is a description of the output.
 	Description *String
 	// Value is an expression for the value of the output.
@@ -270,16 +276,16 @@ type WorkflowCallEventOutput struct {
 // WorkflowCallEvent is workflow_call event configuration.
 // https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#workflow-reuse-events
 type WorkflowCallEvent struct {
-	// Inputs is a map from input name to input configuration.
+	// Inputs is a map from input name to input configuration. Keys are in lower case since they are case-insensitive.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callinputs
-	Inputs map[*String]*WorkflowCallEventInput
+	Inputs map[string]*WorkflowCallEventInput
 	// Secrets is a map from name of secret to secret configuration. When 'secrets' is omitted, nil is set to this
-	// field.
+	// field. Keys are in lower case since they are case-insensitive.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callsecrets
-	Secrets map[*String]*WorkflowCallEventSecret
-	// Outputs is a map from name of output to output configuration.
+	Secrets map[string]*WorkflowCallEventSecret
+	// Outputs is a map from name of output to output configuration. Keys are in lower case since they are case-insensitive.
 	// https://docs.github.com/en/actions/using-workflows/reusing-workflows#using-outputs-from-a-reusable-workflow
-	Outputs map[*String]*WorkflowCallEventOutput
+	Outputs map[string]*WorkflowCallEventOutput
 	// Pos is a position in source.
 	Pos *Pos
 }
@@ -400,7 +406,7 @@ type Input struct {
 type ExecAction struct {
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepsuses
 	Uses *String
-	// Inputs represents inputs to the action to execute in 'with' section
+	// Inputs represents inputs to the action to execute in 'with' section. Keys are in lower case since they are case-insensitive.
 	Inputs map[string]*Input
 	// Entrypoint represents optional 'entrypoint' field in 'with' section. Nil field means nothing specified
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstepswithentrypoint
@@ -442,7 +448,7 @@ type RawYAMLValue interface {
 
 // RawYAMLObject is raw YAML mapping value.
 type RawYAMLObject struct {
-	// Props is map from property names to their values.
+	// Props is map from property names to their values. Keys are in lower case since they are case-insensitive.
 	Props map[string]RawYAMLValue
 	pos   *Pos
 }
@@ -610,7 +616,7 @@ func (cs *MatrixCombinations) ContainsExpression() bool {
 // Matrix is matrix variations configuration of a job.
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstrategymatrix
 type Matrix struct {
-	// Values stores mappings from name to values.
+	// Values stores mappings from name to values. Keys are in lower case since they are case-insensitive.
 	Rows map[string]*MatrixRow
 	// Include is list of combinations of matrix values and additional values on running matrix combinations.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#example-including-additional-values-into-combinations
@@ -819,7 +825,7 @@ type Job struct {
 	ContinueOnError *Bool
 	// Container is container configuration to run the job.
 	Container *Container
-	// Services is map from service names to service configurations.
+	// Services is map from service names to service configurations. Keys are in lower case since they are case-insensitive.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idservices
 	Services map[string]*Service
 	// WorkflowCall is a workflow call by 'uses:'.
@@ -835,6 +841,9 @@ type Workflow struct {
 	// Name is name of the workflow. This field can be nil when user didn't specify the name explicitly.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#name
 	Name *String
+	// RunName is the name of workflow runs. This field can be set dynamically using ${{ }}.
+	// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#run-name
+	RunName *String
 	// On is list of events which can trigger this workflow.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onpushpull_requestbranchestags
 	On []Event
@@ -848,7 +857,7 @@ type Workflow struct {
 	// Concurrency is concurrency configuration of entire workflow. Each jobs also can their own
 	// concurrency configurations.
 	Concurrency *Concurrency
-	// Jobs is mappings from job ID to the job object
+	// Jobs is mappings from job ID to the job object. Keys are in lower case since they are case-insensitive.
 	Jobs map[string]*Job
 }
 
