@@ -572,9 +572,13 @@ func (p *parser) parseEvents(pos *Pos, n *yaml.Node) []Event {
 				&WorkflowCallEvent{Pos: posAt(n)},
 			}
 		default:
+			h := p.parseString(n, false)
+			if h.Value == "" {
+				return []Event{}
+			}
 			return []Event{
 				&WebhookEvent{
-					Hook: p.parseString(n, false),
+					Hook: h,
 					Pos:  posAt(n),
 				},
 			}
@@ -1262,8 +1266,15 @@ func (p *parser) parseJobs(n *yaml.Node) map[string]*Job {
 func (p *parser) parse(n *yaml.Node) *Workflow {
 	w := &Workflow{}
 
+	if n.Line == 0 {
+		n.Line = 1
+	}
+	if n.Column == 0 {
+		n.Column = 1
+	}
+
 	if len(n.Content) == 0 {
-		p.error(n, "\"jobs\" section is missing in workflow")
+		p.error(n, "workflow is empty")
 		return w
 	}
 
@@ -1289,6 +1300,7 @@ func (p *parser) parse(n *yaml.Node) *Workflow {
 		default:
 			p.unexpectedKey(k, "workflow", []string{
 				"name",
+				"run-name",
 				"on",
 				"permissions",
 				"env",
@@ -1299,6 +1311,9 @@ func (p *parser) parse(n *yaml.Node) *Workflow {
 		}
 	}
 
+	if w.On == nil {
+		p.error(n, "\"on\" section is missing in workflow")
+	}
 	if w.Jobs == nil {
 		p.error(n, "\"jobs\" section is missing in workflow")
 	}
