@@ -6,6 +6,8 @@ import (
 	"strings"
 )
 
+//go:generate go run ./scripts/generate-availability ./availability.go
+
 type typedExpr struct {
 	ty  ExprType
 	pos Pos
@@ -49,29 +51,28 @@ func NewRuleExpression(actionsCache *LocalActionsCache, workflowCache *LocalReus
 
 // VisitWorkflowPre is callback when visiting Workflow node before visiting its children.
 func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
-	rule.checkString(n.Name)
-	rule.checkString(n.RunName)
+	rule.checkString(n.Name, "")
 
 	for _, e := range n.On {
 		switch e := e.(type) {
 		case *WebhookEvent:
-			rule.checkStrings(e.Types)
+			rule.checkStrings(e.Types, "")
 			rule.checkWebhookEventFilter(e.Branches)
 			rule.checkWebhookEventFilter(e.BranchesIgnore)
 			rule.checkWebhookEventFilter(e.Tags)
 			rule.checkWebhookEventFilter(e.TagsIgnore)
 			rule.checkWebhookEventFilter(e.Paths)
 			rule.checkWebhookEventFilter(e.PathsIgnore)
-			rule.checkStrings(e.Workflows)
+			rule.checkStrings(e.Workflows, "")
 		case *ScheduledEvent:
-			rule.checkStrings(e.Cron)
+			rule.checkStrings(e.Cron, "")
 		case *WorkflowDispatchEvent:
 			ity := NewEmptyStrictObjectType()
-			for n, i := range e.Inputs {
-				rule.checkString(i.Description)
-				rule.checkString(i.Default)
-				rule.checkBool(i.Required)
-				rule.checkStrings(i.Options)
+			for id, i := range e.Inputs {
+				rule.checkString(i.Description, "")
+				rule.checkString(i.Default, "")
+				rule.checkBool(i.Required, "")
+				rule.checkStrings(i.Options, "")
 
 				var ty ExprType
 				switch i.Type {
@@ -82,11 +83,11 @@ func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
 				default:
 					ty = AnyType{}
 				}
-				ity.Props[n] = ty
+				ity.Props[id] = ty
 			}
 			rule.dispatchInputsTy = ity
 		case *RepositoryDispatchEvent:
-			rule.checkStrings(e.Types)
+			rule.checkStrings(e.Types, "")
 		case *WorkflowCallEvent:
 			ity := NewEmptyStrictObjectType()
 			for n, i := range e.Inputs {
@@ -103,8 +104,8 @@ func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
 				}
 				ity.Props[n] = ty
 
-				rule.checkString(i.Description)
-				rule.checkString(i.Default)
+				rule.checkString(i.Description, "")
+				rule.checkString(i.Default, "on.workflow_call.inputs.<inputs_id>.default")
 			}
 			rule.inputsTy = ity
 
@@ -113,23 +114,25 @@ func (rule *RuleExpression) VisitWorkflowPre(n *Workflow) error {
 			// exist. When `e.Secrets` is an empty map, `secrets:` exists but it has no child.
 			if e.Secrets != nil {
 				sty := NewEmptyStrictObjectType()
-				for n, s := range e.Secrets {
-					sty.Props[n] = StringType{}
-					rule.checkString(s.Description)
+				for id, s := range e.Secrets {
+					sty.Props[id] = StringType{}
+					rule.checkString(s.Description, "")
 				}
 				rule.secretsTy = sty
 			}
 
 			for _, o := range e.Outputs {
-				rule.checkString(o.Description)
+				rule.checkString(o.Description, "")
+				// o.Value will be checked in VisitWorkflowPost
 			}
 		}
 	}
 
-	rule.checkEnv(n.Env, true)
+	rule.checkString(n.RunName, "run-name")
+	rule.checkEnv(n.Env, "env")
 
-	rule.checkDefaults(n.Defaults)
-	rule.checkConcurrency(n.Concurrency)
+	rule.checkDefaults(n.Defaults, "")
+	rule.checkConcurrency(n.Concurrency, "concurrency")
 
 	rule.workflow = n
 	return nil
@@ -162,12 +165,12 @@ func (rule *RuleExpression) VisitJobPre(n *Job) error {
 		rule.matrixTy = rule.guessTypeOfMatrix(n.Strategy.Matrix)
 	}
 
-	rule.checkString(n.Name)
-	rule.checkStrings(n.Needs)
+	rule.checkString(n.Name, "jobs.<job_id>.name")
+	rule.checkStrings(n.Needs, "")
 
 	if n.RunsOn != nil {
 		if n.RunsOn.Expression != nil {
-			if ty := rule.checkOneExpression(n.RunsOn.Expression, "runner label at \"runs-on\" section"); ty != nil {
+			if ty := rule.checkOneExpression(n.RunsOn.Expression, "runner label at \"runs-on\" section", "jobs.<job_id>.runs-on"); ty != nil {
 				switch ty.(type) {
 				case *ArrayType, StringType, AnyType:
 					// OK
@@ -177,17 +180,17 @@ func (rule *RuleExpression) VisitJobPre(n *Job) error {
 			}
 		} else {
 			for _, l := range n.RunsOn.Labels {
-				rule.checkString(l)
+				rule.checkString(l, "jobs.<job_id>.runs-on")
 			}
 		}
 	}
 
-	rule.checkConcurrency(n.Concurrency)
+	rule.checkConcurrency(n.Concurrency, "jobs.<job_id>.concurrency")
 
-	rule.checkEnv(n.Env, true)
+	rule.checkEnv(n.Env, "jobs.<job_id>.env")
 
-	rule.checkDefaults(n.Defaults)
-	rule.checkIfCondition(n.If, true)
+	rule.checkDefaults(n.Defaults, "jobs.<job_id>.defaults.run")
+	rule.checkIfCondition(n.If, "jobs.<job_id>.if")
 
 	if n.Strategy != nil {
 		if n.Strategy.Matrix != nil {
@@ -199,16 +202,16 @@ func (rule *RuleExpression) VisitJobPre(n *Job) error {
 			rule.checkMatrixCombinations(n.Strategy.Matrix.Include, "include")
 			rule.checkMatrixCombinations(n.Strategy.Matrix.Exclude, "exclude")
 		}
-		rule.checkBool(n.Strategy.FailFast)
-		rule.checkInt(n.Strategy.MaxParallel)
+		rule.checkBool(n.Strategy.FailFast, "jobs.<job_id>.strategy")
+		rule.checkInt(n.Strategy.MaxParallel, "jobs.<job_id>.strategy")
 	}
 
-	rule.checkBool(n.ContinueOnError)
-	rule.checkFloat(n.TimeoutMinutes)
-	rule.checkContainer(n.Container)
+	rule.checkBool(n.ContinueOnError, "jobs.<job_id>.continue-on-error")
+	rule.checkFloat(n.TimeoutMinutes, "jobs.<job_id>.timeout-minutes")
+	rule.checkContainer(n.Container, "jobs.<job_id>.container", "")
 
 	for _, s := range n.Services {
-		rule.checkContainer(s.Container)
+		rule.checkContainer(s.Container, "jobs.<job_id>.services", "<service_id>")
 	}
 
 	rule.checkWorkflowCall(n.WorkflowCall)
@@ -222,11 +225,11 @@ func (rule *RuleExpression) VisitJobPre(n *Job) error {
 func (rule *RuleExpression) VisitJobPost(n *Job) error {
 	// 'environment' and 'outputs' sections are evaluated after all steps are run
 	if n.Environment != nil {
-		rule.checkString(n.Environment.Name)
-		rule.checkString(n.Environment.URL)
+		rule.checkString(n.Environment.Name, "jobs.<job_id>.environment")
+		rule.checkString(n.Environment.URL, "jobs.<job_id>.environment.url")
 	}
 	for _, output := range n.Outputs {
-		rule.checkString(output.Value)
+		rule.checkString(output.Value, "jobs.<job_id>.outputs.<output_id>")
 	}
 
 	rule.matrixTy = nil
@@ -238,38 +241,38 @@ func (rule *RuleExpression) VisitJobPost(n *Job) error {
 
 // VisitStep is callback when visiting Step node.
 func (rule *RuleExpression) VisitStep(n *Step) error {
-	rule.checkIfCondition(n.If, false)
-	rule.checkString(n.Name)
+	rule.checkString(n.Name, "jobs.<job_id>.steps.name")
+	rule.checkIfCondition(n.If, "jobs.<job_id>.steps.if")
 
 	var spec *String
 	switch e := n.Exec.(type) {
 	case *ExecRun:
-		rule.checkScriptString(e.Run)
-		rule.checkString(e.Shell)
-		rule.checkString(e.WorkingDirectory)
+		rule.checkScriptString(e.Run, "jobs.<job_id>.steps.run")
+		rule.checkString(e.Shell, "")
+		rule.checkString(e.WorkingDirectory, "jobs.<job_id>.steps.working-directory")
 	case *ExecAction:
-		rule.checkStringNoEnv(e.Uses)
+		rule.checkString(e.Uses, "")
 		for n, i := range e.Inputs {
 			if e.Uses != nil && strings.HasPrefix(e.Uses.Value, "actions/github-script@") && n == "script" {
-				rule.checkScriptString(i.Value)
+				rule.checkScriptString(i.Value, "jobs.<job_id>.steps.with")
 			} else {
-				rule.checkString(i.Value)
+				rule.checkString(i.Value, "jobs.<job_id>.steps.with")
 			}
 		}
-		rule.checkString(e.Entrypoint)
-		rule.checkString(e.Args)
+		rule.checkString(e.Entrypoint, "")
+		rule.checkString(e.Args, "")
 		spec = e.Uses
 	}
 
-	rule.checkEnv(n.Env, false) // env: at step level can refer 'env' context (#158)
-	rule.checkBool(n.ContinueOnError)
-	rule.checkFloat(n.TimeoutMinutes)
+	rule.checkEnv(n.Env, "jobs.<job_id>.steps.env") // env: at step level can refer 'env' context (#158)
+	rule.checkBool(n.ContinueOnError, "jobs.<job_id>.steps.continue-on-error")
+	rule.checkFloat(n.TimeoutMinutes, "jobs.<job_id>.steps.timeout-minutes")
 
 	if n.ID != nil {
 		// Step ID is case insensitive
 		id := strings.ToLower(n.ID.Value)
 		if strings.Contains(id, "${{") && strings.Contains(id, "}}") {
-			rule.checkStringNoEnv(n.ID)
+			rule.checkString(n.ID, "")
 			rule.stepsTy.Loose()
 		}
 		rule.stepsTy.Props[id] = NewStrictObjectType(map[string]ExprType{
@@ -337,13 +340,13 @@ func (rule *RuleExpression) getWorkflowCallOutputsType(call *WorkflowCall) *Obje
 	return NewStrictObjectType(p)
 }
 
-func (rule *RuleExpression) checkOneExpression(s *String, what string) ExprType {
+func (rule *RuleExpression) checkOneExpression(s *String, what, workflowKey string) ExprType {
 	// checkString is not available since it checks types for embedding values into a string
 	if s == nil {
 		return nil
 	}
 
-	ts, ok := rule.checkExprsIn(s.Value, s.Pos, s.Quoted, false, false)
+	ts, ok := rule.checkExprsIn(s.Value, s.Pos, s.Quoted, false, workflowKey)
 	if !ok {
 		return nil
 	}
@@ -396,24 +399,24 @@ func (rule *RuleExpression) checkNumberTy(ty ExprType, pos *Pos, what string) Ex
 	}
 }
 
-func (rule *RuleExpression) checkObjectExpression(s *String, what string) ExprType {
-	ty := rule.checkOneExpression(s, what)
+func (rule *RuleExpression) checkObjectExpression(s *String, what, workflowKey string) ExprType {
+	ty := rule.checkOneExpression(s, what, workflowKey)
 	if ty == nil {
 		return nil
 	}
 	return rule.checkObjectTy(ty, s.Pos, what)
 }
 
-func (rule *RuleExpression) checkArrayExpression(s *String, what string) ExprType {
-	ty := rule.checkOneExpression(s, what)
+func (rule *RuleExpression) checkArrayExpression(s *String, what, workflowKey string) ExprType {
+	ty := rule.checkOneExpression(s, what, workflowKey)
 	if ty == nil {
 		return nil
 	}
 	return rule.checkArrayTy(ty, s.Pos, what)
 }
 
-func (rule *RuleExpression) checkNumberExpression(s *String, what string) ExprType {
-	ty := rule.checkOneExpression(s, what)
+func (rule *RuleExpression) checkNumberExpression(s *String, what, workflowKey string) ExprType {
+	ty := rule.checkOneExpression(s, what, workflowKey)
 	if ty == nil {
 		return nil
 	}
@@ -426,7 +429,7 @@ func (rule *RuleExpression) checkMatrixCombinations(cs *MatrixCombinations, what
 	}
 
 	if cs.Expression != nil {
-		if ty, ok := rule.checkArrayExpression(cs.Expression, what).(*ArrayType); ok {
+		if ty, ok := rule.checkArrayExpression(cs.Expression, what, "jobs.<job_id>.strategy").(*ArrayType); ok {
 			rule.checkObjectTy(ty.Elem, cs.Expression.Pos, what)
 		}
 		return
@@ -435,7 +438,7 @@ func (rule *RuleExpression) checkMatrixCombinations(cs *MatrixCombinations, what
 	what = fmt.Sprintf("matrix combination at element of %s section", what)
 	for _, combi := range cs.Combinations {
 		if combi.Expression != nil {
-			rule.checkObjectExpression(combi.Expression, what)
+			rule.checkObjectExpression(combi.Expression, what, "jobs.<job_id>.strategy")
 			continue
 		}
 		for _, a := range combi.Assigns {
@@ -444,55 +447,56 @@ func (rule *RuleExpression) checkMatrixCombinations(cs *MatrixCombinations, what
 	}
 }
 
-func (rule *RuleExpression) checkEnv(env *Env, noEnv bool) {
+func (rule *RuleExpression) checkEnv(env *Env, workflowKey string) {
 	if env == nil {
 		return
 	}
 
 	if env.Vars != nil {
 		for _, e := range env.Vars {
-			if noEnv {
-				rule.checkStringNoEnv(e.Value)
-			} else {
-				rule.checkString(e.Value)
-			}
+			rule.checkString(e.Value, workflowKey)
 		}
 		return
 	}
 
 	// When form of "env: ${{...}}"
-	rule.checkObjectExpression(env.Expression, "env")
+	rule.checkObjectExpression(env.Expression, "env", workflowKey)
 }
 
-func (rule *RuleExpression) checkContainer(c *Container) {
+func (rule *RuleExpression) checkContainer(c *Container, workflowKey, childWorkflowKeyPrefix string) {
 	if c == nil {
 		return
 	}
-	rule.checkString(c.Image)
+	childWorkflowKey := workflowKey
+	if childWorkflowKeyPrefix != "" {
+		childWorkflowKey += "." + childWorkflowKeyPrefix
+	}
+	rule.checkString(c.Image, workflowKey)
 	if c.Credentials != nil {
-		rule.checkString(c.Credentials.Username)
-		rule.checkString(c.Credentials.Password)
+		k := childWorkflowKey + ".credentials" // e.g. jobs.<job_id>.container.credentials
+		rule.checkString(c.Credentials.Username, k)
+		rule.checkString(c.Credentials.Password, k)
 	}
-	rule.checkEnv(c.Env, false)
-	rule.checkStrings(c.Ports)
-	rule.checkStrings(c.Volumes)
-	rule.checkString(c.Options)
+	rule.checkEnv(c.Env, workflowKey+".env.<env_id>") // e.g. jobs.<job_id>.container.env.<env_id>
+	rule.checkStrings(c.Ports, workflowKey)
+	rule.checkStrings(c.Volumes, workflowKey)
+	rule.checkString(c.Options, workflowKey)
 }
 
-func (rule *RuleExpression) checkConcurrency(c *Concurrency) {
+func (rule *RuleExpression) checkConcurrency(c *Concurrency, workflowKey string) {
 	if c == nil {
 		return
 	}
-	rule.checkString(c.Group)
-	rule.checkBool(c.CancelInProgress)
+	rule.checkString(c.Group, workflowKey)
+	rule.checkBool(c.CancelInProgress, workflowKey)
 }
 
-func (rule *RuleExpression) checkDefaults(d *Defaults) {
+func (rule *RuleExpression) checkDefaults(d *Defaults, workflowKey string) {
 	if d == nil || d.Run == nil {
 		return
 	}
-	rule.checkString(d.Run.Shell)
-	rule.checkString(d.Run.WorkingDirectory)
+	rule.checkString(d.Run.Shell, workflowKey)
+	rule.checkString(d.Run.WorkingDirectory, workflowKey)
 }
 
 func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
@@ -500,7 +504,7 @@ func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
 		return
 	}
 
-	rule.checkStringNoEnv(c.Uses)
+	rule.checkString(c.Uses, "")
 
 	m, err := rule.localWorkflows.FindMetadata(c.Uses.Value)
 	if err != nil {
@@ -508,7 +512,7 @@ func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
 	}
 
 	for n, i := range c.Inputs {
-		ts := rule.checkString(i.Value)
+		ts := rule.checkString(i.Value, "jobs.<job_id>.with.<with_id>")
 
 		if m == nil {
 			continue
@@ -556,7 +560,7 @@ func (rule *RuleExpression) checkWorkflowCall(c *WorkflowCall) {
 	}
 
 	for _, s := range c.Secrets {
-		rule.checkString(s.Value)
+		rule.checkString(s.Value, "jobs.<job_id>.secrets.<secrets_id>")
 	}
 }
 
@@ -564,16 +568,16 @@ func (rule *RuleExpression) checkWebhookEventFilter(f *WebhookEventFilter) {
 	if f == nil {
 		return
 	}
-	rule.checkStrings(f.Values)
+	rule.checkStrings(f.Values, "")
 }
 
-func (rule *RuleExpression) checkStrings(ss []*String) {
+func (rule *RuleExpression) checkStrings(ss []*String, workflowKey string) {
 	for _, s := range ss {
-		rule.checkString(s)
+		rule.checkString(s, workflowKey)
 	}
 }
 
-func (rule *RuleExpression) checkIfCondition(str *String, noEnv bool) {
+func (rule *RuleExpression) checkIfCondition(str *String, workflowKey string) {
 	if str == nil {
 		return
 	}
@@ -602,13 +606,7 @@ func (rule *RuleExpression) checkIfCondition(str *String, noEnv bool) {
 
 	var condTy ExprType
 	if strings.Contains(str.Value, "${{") && strings.Contains(str.Value, "}}") {
-		var ts []typedExpr
-
-		if noEnv {
-			ts = rule.checkStringNoEnv(str)
-		} else {
-			ts = rule.checkString(str)
-		}
+		ts := rule.checkString(str, workflowKey)
 
 		if len(ts) == 1 {
 			s := strings.TrimSpace(str.Value)
@@ -627,7 +625,7 @@ func (rule *RuleExpression) checkIfCondition(str *String, noEnv bool) {
 			return
 		}
 
-		if ty, ok := rule.checkSemanticsOfExprNode(expr, line, col, false, noEnv); ok {
+		if ty, ok := rule.checkSemanticsOfExprNode(expr, line, col, false, workflowKey); ok {
 			condTy = ty
 		}
 	}
@@ -646,12 +644,12 @@ func (rule *RuleExpression) checkTemplateEvaluatedType(ts []typedExpr) {
 	}
 }
 
-func (rule *RuleExpression) checkString(str *String) []typedExpr {
+func (rule *RuleExpression) checkString(str *String, workflowKey string) []typedExpr {
 	if str == nil {
 		return nil
 	}
 
-	ts, ok := rule.checkExprsIn(str.Value, str.Pos, str.Quoted, false, false)
+	ts, ok := rule.checkExprsIn(str.Value, str.Pos, str.Quoted, false, workflowKey)
 	if !ok {
 		return nil
 	}
@@ -660,12 +658,12 @@ func (rule *RuleExpression) checkString(str *String) []typedExpr {
 	return ts
 }
 
-func (rule *RuleExpression) checkScriptString(str *String) {
+func (rule *RuleExpression) checkScriptString(str *String, workflowKey string) {
 	if str == nil {
 		return
 	}
 
-	ts, ok := rule.checkExprsIn(str.Value, str.Pos, str.Quoted, true, false)
+	ts, ok := rule.checkExprsIn(str.Value, str.Pos, str.Quoted, true, workflowKey)
 	if !ok {
 		return
 	}
@@ -673,34 +671,12 @@ func (rule *RuleExpression) checkScriptString(str *String) {
 	rule.checkTemplateEvaluatedType(ts)
 }
 
-// When checking 'id:' or 'uses:' or 'env:' at toplevel or 'env:' at job level, 'env' context cannot
-// be referred. (#158)
-//
-// > Variables in the env map cannot be defined in terms of other variables in the map.
-// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#env
-//
-// > You can use the env context in the value of any key in a step except for the id and uses keys.
-// https://docs.github.com/en/actions/learn-github-actions/contexts#env-context
-func (rule *RuleExpression) checkStringNoEnv(str *String) []typedExpr {
-	if str == nil {
-		return nil
-	}
-
-	ts, ok := rule.checkExprsIn(str.Value, str.Pos, str.Quoted, false, true)
-	if !ok {
-		return nil
-	}
-
-	rule.checkTemplateEvaluatedType(ts)
-	return ts
-}
-
-func (rule *RuleExpression) checkBool(b *Bool) {
+func (rule *RuleExpression) checkBool(b *Bool, workflowKey string) {
 	if b == nil || b.Expression == nil {
 		return
 	}
 
-	ty := rule.checkOneExpression(b.Expression, "bool value")
+	ty := rule.checkOneExpression(b.Expression, "bool value", workflowKey)
 	if ty == nil {
 		return
 	}
@@ -713,21 +689,21 @@ func (rule *RuleExpression) checkBool(b *Bool) {
 	}
 }
 
-func (rule *RuleExpression) checkInt(i *Int) {
+func (rule *RuleExpression) checkInt(i *Int, workflowKey string) {
 	if i == nil {
 		return
 	}
-	rule.checkNumberExpression(i.Expression, "integer value")
+	rule.checkNumberExpression(i.Expression, "integer value", workflowKey)
 }
 
-func (rule *RuleExpression) checkFloat(f *Float) {
+func (rule *RuleExpression) checkFloat(f *Float, workflowKey string) {
 	if f == nil {
 		return
 	}
-	rule.checkNumberExpression(f.Expression, "float number value")
+	rule.checkNumberExpression(f.Expression, "float number value", workflowKey)
 }
 
-func (rule *RuleExpression) checkExprsIn(s string, pos *Pos, quoted bool, checkUntrusted, noEnv bool) ([]typedExpr, bool) {
+func (rule *RuleExpression) checkExprsIn(s string, pos *Pos, quoted, checkUntrusted bool, workflowKey string) ([]typedExpr, bool) {
 	// TODO: Line number is not correct when the string contains newlines.
 
 	line, col := pos.Line, pos.Col
@@ -747,7 +723,7 @@ func (rule *RuleExpression) checkExprsIn(s string, pos *Pos, quoted bool, checkU
 		offset += start
 		col := col + offset
 
-		ty, offsetAfter, ok := rule.checkSemantics(s, line, col, checkUntrusted, noEnv)
+		ty, offsetAfter, ok := rule.checkSemantics(s, line, col, checkUntrusted, workflowKey)
 		if !ok {
 			return nil, false
 		}
@@ -774,7 +750,7 @@ func (rule *RuleExpression) checkRawYAMLValue(v RawYAMLValue) {
 			rule.checkRawYAMLValue(v)
 		}
 	case *RawYAMLString:
-		rule.checkExprsIn(v.Value, v.Pos(), false, false, false)
+		rule.checkExprsIn(v.Value, v.Pos(), false, false, "jobs.<job_id>.strategy")
 	default:
 		panic("unreachable")
 	}
@@ -785,7 +761,7 @@ func (rule *RuleExpression) exprError(err *ExprError, lineBase, colBase int) {
 	rule.error(pos, err.Message)
 }
 
-func (rule *RuleExpression) checkSemanticsOfExprNode(expr ExprNode, line, col int, checkUntrusted, noEnv bool) (ExprType, bool) {
+func (rule *RuleExpression) checkSemanticsOfExprNode(expr ExprNode, line, col int, checkUntrusted bool, workflowKey string) (ExprType, bool) {
 	c := NewExprSemanticsChecker(checkUntrusted)
 	if rule.matrixTy != nil {
 		c.UpdateMatrix(rule.matrixTy)
@@ -808,8 +784,13 @@ func (rule *RuleExpression) checkSemanticsOfExprNode(expr ExprNode, line, col in
 	if rule.jobsTy != nil {
 		c.UpdateJobs(rule.jobsTy)
 	}
-	if noEnv {
-		c.NoEnv()
+	if workflowKey != "" {
+		ctx, sp := WorkflowKeyAvailability(workflowKey)
+		if len(ctx) == 0 {
+			rule.debug("No context avaiability was found for workflow key %q", workflowKey)
+		}
+		c.SetContextAvailability(ctx)
+		c.SetSpecialFunctionAvailability(sp)
 	}
 
 	ty, errs := c.Check(expr)
@@ -820,7 +801,7 @@ func (rule *RuleExpression) checkSemanticsOfExprNode(expr ExprNode, line, col in
 	return ty, len(errs) == 0
 }
 
-func (rule *RuleExpression) checkSemantics(src string, line, col int, checkUntrusted, noEnv bool) (ExprType, int, bool) {
+func (rule *RuleExpression) checkSemantics(src string, line, col int, checkUntrusted bool, workflowKey string) (ExprType, int, bool) {
 	l := NewExprLexer(src)
 	p := NewExprParser()
 	expr, err := p.Parse(l)
@@ -828,7 +809,7 @@ func (rule *RuleExpression) checkSemantics(src string, line, col int, checkUntru
 		rule.exprError(err, line, col)
 		return nil, l.Offset(), false
 	}
-	t, ok := rule.checkSemanticsOfExprNode(expr, line, col, checkUntrusted, noEnv)
+	t, ok := rule.checkSemanticsOfExprNode(expr, line, col, checkUntrusted, workflowKey)
 	return t, l.Offset(), ok
 }
 
@@ -874,7 +855,7 @@ func (rule *RuleExpression) populateDependantNeedsTypes(out *ObjectType, job *Jo
 }
 
 func (rule *RuleExpression) guessTypeOfMatrixExpression(expr *String) *ObjectType {
-	ty := rule.checkObjectExpression(expr, "matrix")
+	ty := rule.checkObjectExpression(expr, "matrix", "jobs.<job_id>.strategy")
 	if ty == nil {
 		return NewEmptyObjectType()
 	}
@@ -924,7 +905,7 @@ func (rule *RuleExpression) guessTypeOfMatrix(m *Matrix) *ObjectType {
 	}
 
 	if m.Include.Expression != nil {
-		if a, ok := rule.checkOneExpression(m.Include.Expression, "include").(*ArrayType); ok {
+		if a, ok := rule.checkOneExpression(m.Include.Expression, "include", "jobs.<job_id>.strategy").(*ArrayType); ok {
 			if ret, ok := o.Merge(a.Elem).(*ObjectType); ok {
 				return ret
 			}
@@ -934,7 +915,7 @@ func (rule *RuleExpression) guessTypeOfMatrix(m *Matrix) *ObjectType {
 
 	for _, combi := range m.Include.Combinations {
 		if combi.Expression != nil {
-			ty := rule.checkOneExpression(m.Include.Expression, "matrix combination at element of include section")
+			ty := rule.checkOneExpression(m.Include.Expression, "matrix combination at element of include section", "jobs.<job_id>.strategy")
 			if ty == nil {
 				continue
 			}
@@ -963,7 +944,7 @@ func (rule *RuleExpression) guessTypeOfMatrix(m *Matrix) *ObjectType {
 
 func (rule *RuleExpression) guessTypeOfMatrixRow(r *MatrixRow) ExprType {
 	if r.Expression != nil {
-		if a, ok := rule.checkArrayExpression(r.Expression, "matrix row").(*ArrayType); ok {
+		if a, ok := rule.checkArrayExpression(r.Expression, "matrix row", "jobs.<job_id>.strategy").(*ArrayType); ok {
 			return a
 		}
 		return AnyType{}
@@ -1011,7 +992,7 @@ func (rule *RuleExpression) checkWorkflowCallOutputs(outputs map[string]*Workflo
 	rule.jobsTy = NewStrictObjectType(props)
 
 	for _, o := range outputs {
-		rule.checkString(o.Value)
+		rule.checkString(o.Value, "on.workflow_call.outputs.<output_id>.value")
 	}
 }
 
